/*******************************************************************************

  Copyright(c) 1999 - 2015 Intel Corporation. All rights reserved.
  
  This program is free software; you can redistribute it and/or modify it 
  under the terms of the GNU General Public License as published by the Free 
  Software Foundation; either version 2 of the License, or (at your option) 
  any later version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
  more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 
  Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
  The full GNU General Public License is included in this distribution in the
  file called LICENSE.
  
  Contact Information:
  e1000 mailing list <e1000-devel@lists.sourceforge.net>
  Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497

*******************************************************************************/

#include <stdio.h>
#include "ethregs.h"


extern int clr;
extern int offset;

static u32 e1000_translate_register_82542(u32 reg);

#define printit32_82542(i, name, mem) \
		{	u32 i_82542 = e1000_translate_register_82542((u32)i); \
			if (offset) \
				printf("\t(0x%05x) %-15s%08x\n", i_82542, name, \
						*((u32 *)(mem + i_82542))); \
			else \
				printf("\t%-15s%08x\n", name, *((u32 *)(mem + i_82542))); \
		}

void dump_82542_registers(volatile void *mem, u32 size)
{
	unsigned int i, j, n;
	char name[16];

	for (i = 0; i < size; i += sizeof(u32)) {
		switch (i & 0xffffffff) {
		case E1000_CTRL:
			sprintf(name, "CTRL"); break;
		case E1000_STATUS:
			sprintf(name, "STATUS"); break;
		case E1000_EECD:
			sprintf(name, "EECD"); break;
		case E1000_FCAL:
			sprintf(name, "FCAL"); break;
		case E1000_FCT:
			sprintf(name, "FCT"); break;
		case E1000_VET:
			sprintf(name, "VET"); break;
		case E1000_RA:
			for (j=i, n=0; n<16; n++) {
				sprintf(name, "RAL[%d]", n);
				printit32_82542(j, name, mem);
				j += sizeof(u32);
				sprintf(name, "RAH[%d]", n);
				printit32_82542(j, name, mem);
				j += sizeof(u32);
			}
			continue;
		case E1000_ICR:
			if (!clr) continue;
			sprintf(name, "ICR"); break;
		case E1000_ICS:
			sprintf(name, "ICS"); break;
		case E1000_IMS:
			sprintf(name, "IMS"); break;
		case E1000_IMC:
			sprintf(name, "IMC"); break;
		case E1000_RCTL:
			sprintf(name, "RCTL"); break;
		case E1000_RDTR:
			sprintf(name, "RDTR"); break;
		case E1000_RDBAL(0):
			sprintf(name, "RDBAL0"); break;
		case E1000_RDBAH(0):
			sprintf(name, "RDBAH0"); break;
		case E1000_RDLEN(0):
			sprintf(name, "RDLEN0"); break;
		case E1000_RDH(0):
			sprintf(name, "RDH0"); break;
		case E1000_RDT(0):
			sprintf(name, "RDT0"); break;
		case E1000_RDBAL(1):
			sprintf(name, "RDBAL1"); break;
		case E1000_RDBAH(1):
			sprintf(name, "RDBAH1"); break;
		case E1000_RDLEN(1):
			sprintf(name, "RDLEN1"); break;
		case E1000_RDH(1):
			sprintf(name, "RDH1"); break;
		case E1000_RDT(1):
			sprintf(name, "RDT1"); break;
		case E1000_FCRTH:
			sprintf(name, "FCRTH"); break;
		case E1000_FCRTL:
			sprintf(name, "FCRTL"); break;
		case E1000_FCTTV:
			sprintf(name, "FCTTV"); break;
		case E1000_TXCW:
			sprintf(name, "TXCW"); break;
		case E1000_RXCW:
			sprintf(name, "RXCW"); break;
		case E1000_MTA:
			for (j=i, n=0; n<128; n++) {
				sprintf(name, "MTA[%d]", n);
				printit32_82542(j, name, mem);
				j += sizeof(u32);
			}
			continue;
		case E1000_TCTL:
			sprintf(name, "TCTL"); break;
		case E1000_TIPG:
			sprintf(name, "TIPG"); break;
		case E1000_TDBAL(0):
			sprintf(name, "TDBAL"); break;
		case E1000_TDBAH(0):
			sprintf(name, "TDBAH"); break;
		case E1000_TDLEN(0):
			sprintf(name, "TDLEN"); break;
		case E1000_TDH(0):
			sprintf(name, "TDH"); break;
		case E1000_TDT(0):
			sprintf(name, "TDT"); break;
		case E1000_TIDV:
			sprintf(name, "TIDV"); break;
		case E1000_VFTA:
			for (j=i, n=0; n<128; n++) {
				sprintf(name, "VFTA[%d]", n);
				printit32_82542(j, name, mem);
				j += sizeof(u32);
			}
			continue;
		case E1000_CRCERRS:
			if (!clr) continue;
			sprintf(name, "CRCERRS"); break;
		case E1000_SYMERRS:
			if (!clr) continue;
			sprintf(name, "SYMERRS"); break;
		case E1000_MPC:
			if (!clr) continue;
			sprintf(name, "MPC"); break;
		case E1000_SCC:
			if (!clr) continue;
			sprintf(name, "SCC"); break;
		case E1000_ECOL:
			if (!clr) continue;
			sprintf(name, "ECOL"); break;
		case E1000_MCC:
			if (!clr) continue;
			sprintf(name, "MCC"); break;
		case E1000_LATECOL:
			if (!clr) continue;
			sprintf(name, "LATECOL"); break;
		case E1000_COLC:
			if (!clr) continue;
			sprintf(name, "COLC"); break;
		case E1000_DC:
			if (!clr) continue;
			sprintf(name, "DC"); break;
		case E1000_SEC:
			if (!clr) continue;
			sprintf(name, "SEC"); break;
		case E1000_RLEC:
			if (!clr) continue;
			sprintf(name, "RLEC"); break;
		case E1000_XONRXC:
			if (!clr) continue;
			sprintf(name, "XONRXC"); break;
		case E1000_XONTXC:
			if (!clr) continue;
			sprintf(name, "XONTXC"); break;
		case E1000_XOFFRXC:
			if (!clr) continue;
			sprintf(name, "XOFFRXC"); break;
		case E1000_XOFFTXC:
			if (!clr) continue;
			sprintf(name, "XOFFTXC"); break;
		case E1000_FCRUC:
			if (!clr) continue;
			sprintf(name, "FCRUC"); break;
		case E1000_PRC64:
			if (!clr) continue;
			sprintf(name, "PRC64"); break;
		case E1000_PRC127:
			if (!clr) continue;
			sprintf(name, "PRC127"); break;
		case E1000_PRC255:
			if (!clr) continue;
			sprintf(name, "PRC255"); break;
		case E1000_PRC511:
			if (!clr) continue;
			sprintf(name, "PRC511"); break;
		case E1000_PRC1023:
			if (!clr) continue;
			sprintf(name, "PRC1023"); break;
		case E1000_PRC1522:
			if (!clr) continue;
			sprintf(name, "PRC1522"); break;
		case E1000_GPRC:
			if (!clr) continue;
			sprintf(name, "GPRC"); break;
		case E1000_BPRC:
			if (!clr) continue;
			sprintf(name, "BPRC"); break;
		case E1000_MPRC:
			if (!clr) continue;
			sprintf(name, "MPRC"); break;
		case E1000_GPTC:
			if (!clr) continue;
			sprintf(name, "GPTC"); break;
		case E1000_GORCL:
			if (!clr) continue;
			sprintf(name, "GORCL"); break;
		case E1000_GORCH:
			if (!clr) continue;
			sprintf(name, "GORCH"); break;
		case E1000_GOTCL:
			if (!clr) continue;
			sprintf(name, "GOTCL"); break;
		case E1000_GOTCH:
			if (!clr) continue;
			sprintf(name, "GOTCH"); break;
		case E1000_RNBC:
			if (!clr) continue;
			sprintf(name, "RNBC"); break;
		case E1000_RUC:
			if (!clr) continue;
			sprintf(name, "RUC"); break;
		case E1000_RFC:
			if (!clr) continue;
			sprintf(name, "RFC"); break;
		case E1000_ROC:
			if (!clr) continue;
			sprintf(name, "ROC"); break;
		case E1000_RJC:
			if (!clr) continue;
			sprintf(name, "RJC"); break;
		case E1000_TORL:
			if (!clr) continue;
			sprintf(name, "TORL"); break;
		case E1000_TORH:
			if (!clr) continue;
			sprintf(name, "TORH"); break;
		case E1000_TOTL:
			if (!clr) continue;
			sprintf(name, "TOTL"); break;
		case E1000_TOTH:
			if (!clr) continue;
			sprintf(name, "TOTH"); break;
		case E1000_TPR:
			if (!clr) continue;
			sprintf(name, "TPR"); break;
		case E1000_TPT:
			if (!clr) continue;
			sprintf(name, "TPT"); break;
		case E1000_PTC64:
			if (!clr) continue;
			sprintf(name, "PTC64"); break;
		case E1000_PTC127:
			if (!clr) continue;
			sprintf(name, "PTC127"); break;
		case E1000_PTC255:
			if (!clr) continue;
			sprintf(name, "PTC255"); break;
		case E1000_PTC511:
			if (!clr) continue;
			sprintf(name, "PTC511"); break;
		case E1000_PTC1023:
			if (!clr) continue;
			sprintf(name, "PTC1023"); break;
		case E1000_PTC1522:
			if (!clr) continue;
			sprintf(name, "PTC1522"); break;
		case E1000_MPTC:
			if (!clr) continue;
			sprintf(name, "MPTC"); break;
		case E1000_BPTC:
			if (!clr) continue;
			sprintf(name, "BPTC"); break;
		case E1000_TDFH:
			sprintf(name, "TDFH"); break;
		case E1000_TDFT:
			sprintf(name, "TDFT"); break;
		default:
			continue;
		}
		printit32_82542(i, name, mem);
	}
}

/**
 *  e1000_translate_register_82542 - Translate the proper register offset
 *  @reg: e1000 register to be read
 *
 *  Registers in 82542 are located in different offsets than other adapters
 *  even though they function in the same manner.  This function takes in
 *  the name of the register to read and returns the correct offset for
 *  82542 silicon.
 **/
static u32 e1000_translate_register_82542(u32 reg)
{
	/*
	 * Some of the 82542 registers are located at different
	 * offsets than they are in newer adapters.
	 * Despite the difference in location, the registers
	 * function in the same manner.
	 */
	switch (reg) {
	case E1000_RA:
		reg = 0x00040;
		break;
	case E1000_RDTR:
		reg = 0x00108;
		break;
	case E1000_RDBAL(0):
		reg = 0x00110;
		break;
	case E1000_RDBAH(0):
		reg = 0x00114;
		break;
	case E1000_RDLEN(0):
		reg = 0x00118;
		break;
	case E1000_RDH(0):
		reg = 0x00120;
		break;
	case E1000_RDT(0):
		reg = 0x00128;
		break;
	case E1000_RDBAL(1):
		reg = 0x00138;
		break;
	case E1000_RDBAH(1):
		reg = 0x0013C;
		break;
	case E1000_RDLEN(1):
		reg = 0x00140;
		break;
	case E1000_RDH(1):
		reg = 0x00148;
		break;
	case E1000_RDT(1):
		reg = 0x00150;
		break;
	case E1000_FCRTH:
		reg = 0x00160;
		break;
	case E1000_FCRTL:
		reg = 0x00168;
		break;
	case E1000_MTA:
		reg = 0x00200;
		break;
	case E1000_TDBAL(0):
		reg = 0x00420;
		break;
	case E1000_TDBAH(0):
		reg = 0x00424;
		break;
	case E1000_TDLEN(0):
		reg = 0x00428;
		break;
	case E1000_TDH(0):
		reg = 0x00430;
		break;
	case E1000_TDT(0):
		reg = 0x00438;
		break;
	case E1000_TIDV:
		reg = 0x00440;
		break;
	case E1000_VFTA:
		reg = 0x00600;
		break;
	case E1000_TDFH:
		reg = 0x08010;
		break;
	case E1000_TDFT:
		reg = 0x08018;
		break;
	default:
		break;
	}

	return reg;
}
// vi:ts=4:sw=4:ht=4:
